@@ -1,25 +1,4 @@
-/*
- * lxc: linux Container library
- *
- * (C) Copyright IBM Corp. 2007, 2008
- *
- * Authors:
- * Daniel Lezcano <daniel.lezcano at free.fr>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
- */
+/* SPDX-License-Identifier: LGPL-2.1+ */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE 1
@@ -37,6 +16,7 @@
 
 #include "config.h"
 #include "log.h"
+#include "macro.h"
 #include "memory_utils.h"
 #include "raw_syscalls.h"
 #include "utils.h"
@@ -48,14 +28,12 @@
 lxc_log_define(af_unix, lxc);
 
 static ssize_t lxc_abstract_unix_set_sockaddr(struct sockaddr_un *addr,
-				const char *path)
+					      const char *path)
 {
 	size_t len;
 
-	if (!addr || !path) {
-		errno = EINVAL;
-		return -1;
-	}
+	if (!addr || !path)
+		return ret_errno(EINVAL);
 
 	/* Clear address structure */
 	memset(addr, 0, sizeof(*addr));
@@ -65,10 +43,8 @@ static ssize_t lxc_abstract_unix_set_sockaddr(struct sockaddr_un *addr,
 	len = strlen(&path[1]);
 
 	/* do not enforce \0-termination */
-	if (len >= INT_MAX || len >= sizeof(addr->sun_path)) {
-		errno = ENAMETOOLONG;
-		return -1;
-	}
+	if (len >= INT_MAX || len >= sizeof(addr->sun_path))
+		return ret_errno(ENAMETOOLONG);
 
 	/* do not enforce \0-termination */
 	memcpy(&addr->sun_path[1], &path[1], len);
@@ -77,7 +53,8 @@ static ssize_t lxc_abstract_unix_set_sockaddr(struct sockaddr_un *addr,
 
 int lxc_abstract_unix_open(const char *path, int type, int flags)
 {
-	int fd, ret;
+	__do_close int fd = -EBADF;
+	int ret;
 	ssize_t len;
 	struct sockaddr_un addr;
 
@@ -86,36 +63,24 @@ int lxc_abstract_unix_open(const char *path, int type, int flags)
 		return -1;
 
 	if (!path)
-		return fd;
+		return move_fd(fd);
 
 	len = lxc_abstract_unix_set_sockaddr(&addr, path);
-	if (len < 0) {
-		int saved_errno = errno;
-		close(fd);
-		errno = saved_errno;
+	if (len < 0)
 		return -1;
-	}
 
 	ret = bind(fd, (struct sockaddr *)&addr,
 		   offsetof(struct sockaddr_un, sun_path) + len + 1);
-	if (ret < 0) {
-		int saved_errno = errno;
-		close(fd);
-		errno = saved_errno;
+	if (ret < 0)
 		return -1;
-	}
 
 	if (type == SOCK_STREAM) {
 		ret = listen(fd, 100);
-		if (ret < 0) {
-			int saved_errno = errno;
-			close(fd);
-			errno = saved_errno;
+		if (ret < 0)
 			return -1;
-		}
 	}
 
-	return fd;
+	return move_fd(fd);
 }
 
 void lxc_abstract_unix_close(int fd)
@@ -125,7 +90,8 @@ void lxc_abstract_unix_close(int fd)
 
 int lxc_abstract_unix_connect(const char *path)
 {
-	int fd, ret;
+	__do_close int fd = -EBADF;
+	int ret;
 	ssize_t len;
 	struct sockaddr_un addr;
 
@@ -134,37 +100,27 @@ int lxc_abstract_unix_connect(const char *path)
 		return -1;
 
 	len = lxc_abstract_unix_set_sockaddr(&addr, path);
-	if (len < 0) {
-		int saved_errno = errno;
-		close(fd);
-		errno = saved_errno;
+	if (len < 0)
 		return -1;
-	}
 
 	ret = connect(fd, (struct sockaddr *)&addr,
 		      offsetof(struct sockaddr_un, sun_path) + len + 1);
-	if (ret < 0) {
-		int saved_errno = errno;
-		close(fd);
-		errno = saved_errno;
+	if (ret < 0)
 		return -1;
-	}
 
-	return fd;
+	return move_fd(fd);
 }
 
-int lxc_abstract_unix_send_fds(int fd, int *sendfds, int num_sendfds,
-			       void *data, size_t size)
+int lxc_abstract_unix_send_fds_iov(int fd, int *sendfds, int num_sendfds,
+				   struct iovec *iov, size_t iovlen)
 {
-	__do_free char *cmsgbuf;
+	__do_free char *cmsgbuf = NULL;
+	int ret;
 	struct msghdr msg;
-	struct iovec iov;
 	struct cmsghdr *cmsg = NULL;
-	char buf[1] = {0};
 	size_t cmsgbufsize = CMSG_SPACE(num_sendfds * sizeof(int));
 
 	memset(&msg, 0, sizeof(msg));
-	memset(&iov, 0, sizeof(iov));
 
 	cmsgbuf = malloc(cmsgbufsize);
 	if (!cmsgbuf) {
@@ -184,51 +140,64 @@ int lxc_abstract_unix_send_fds(int fd, int *sendfds, int num_sendfds,
 
 	memcpy(CMSG_DATA(cmsg), sendfds, num_sendfds * sizeof(int));
 
-	iov.iov_base = data ? data : buf;
-	iov.iov_len = data ? size : sizeof(buf);
-	msg.msg_iov = &iov;
-	msg.msg_iovlen = 1;
+	msg.msg_iov = iov;
+	msg.msg_iovlen = iovlen;
 
-	return sendmsg(fd, &msg, MSG_NOSIGNAL);
+	do {
+		ret = sendmsg(fd, &msg, MSG_NOSIGNAL);
+	} while (ret < 0 && errno == EINTR);
+
+	return ret;
 }
 
-int lxc_abstract_unix_recv_fds(int fd, int *recvfds, int num_recvfds,
+int lxc_abstract_unix_send_fds(int fd, int *sendfds, int num_sendfds,
 			       void *data, size_t size)
 {
-	__do_free char *cmsgbuf;
+	char buf[1] = {0};
+	struct iovec iov = {
+		.iov_base = data ? data : buf,
+		.iov_len = data ? size : sizeof(buf),
+	};
+	return lxc_abstract_unix_send_fds_iov(fd, sendfds, num_sendfds, &iov, 1);
+}
+
+int lxc_unix_send_fds(int fd, int *sendfds, int num_sendfds, void *data,
+		      size_t size)
+{
+	return lxc_abstract_unix_send_fds(fd, sendfds, num_sendfds, data, size);
+}
+
+static int lxc_abstract_unix_recv_fds_iov(int fd, int *recvfds, int num_recvfds,
+					  struct iovec *iov, size_t iovlen)
+{
+	__do_free char *cmsgbuf = NULL;
 	int ret;
 	struct msghdr msg;
-	struct iovec iov;
-	struct cmsghdr *cmsg = NULL;
-	char buf[1] = {0};
 	size_t cmsgbufsize = CMSG_SPACE(sizeof(struct ucred)) +
 			     CMSG_SPACE(num_recvfds * sizeof(int));
 
 	memset(&msg, 0, sizeof(msg));
-	memset(&iov, 0, sizeof(iov));
 
 	cmsgbuf = malloc(cmsgbufsize);
-	if (!cmsgbuf) {
-		errno = ENOMEM;
-		return -1;
-	}
+	if (!cmsgbuf)
+		return ret_errno(ENOMEM);
 
 	msg.msg_control = cmsgbuf;
 	msg.msg_controllen = cmsgbufsize;
 
-	iov.iov_base = data ? data : buf;
-	iov.iov_len = data ? size : sizeof(buf);
-	msg.msg_iov = &iov;
-	msg.msg_iovlen = 1;
+	msg.msg_iov = iov;
+	msg.msg_iovlen = iovlen;
 
-	ret = recvmsg(fd, &msg, 0);
-	if (ret <= 0)
-		goto out;
+	do {
+		ret = recvmsg(fd, &msg, 0);
+	} while (ret < 0 && errno == EINTR);
+	if (ret < 0 || ret == 0)
+		return ret;
 
 	/*
 	 * If SO_PASSCRED is set we will always get a ucred message.
 	 */
-	for (cmsg = CMSG_FIRSTHDR(&msg); cmsg; cmsg = CMSG_NXTHDR(&msg, cmsg)) {
+	for (struct cmsghdr *cmsg = CMSG_FIRSTHDR(&msg); cmsg; cmsg = CMSG_NXTHDR(&msg, cmsg)) {
 		if (cmsg->cmsg_type != SCM_RIGHTS)
 			continue;
 
@@ -240,17 +209,29 @@ int lxc_abstract_unix_recv_fds(int fd, int *recvfds, int num_recvfds,
 		break;
 	}
 
-out:
 	return ret;
 }
 
+int lxc_abstract_unix_recv_fds(int fd, int *recvfds, int num_recvfds,
+			       void *data, size_t size)
+{
+	char buf[1] = {0};
+	struct iovec iov = {
+		.iov_base = data ? data : buf,
+		.iov_len = data ? size : sizeof(buf),
+	};
+	return lxc_abstract_unix_recv_fds_iov(fd, recvfds, num_recvfds, &iov, 1);
+}
+
 int lxc_abstract_unix_send_credential(int fd, void *data, size_t size)
 {
 	struct msghdr msg = {0};
 	struct iovec iov;
 	struct cmsghdr *cmsg;
 	struct ucred cred = {
-	    .pid = lxc_raw_getpid(), .uid = getuid(), .gid = getgid(),
+		.pid = lxc_raw_getpid(),
+		.uid = getuid(),
+		.gid = getgid(),
 	};
 	char cmsgbuf[CMSG_SPACE(sizeof(cred))] = {0};
 	char buf[1] = {0};
@@ -297,7 +278,7 @@ int lxc_abstract_unix_rcv_credential(int fd, void *data, size_t size)
 
 	ret = recvmsg(fd, &msg, 0);
 	if (ret <= 0)
-		goto out;
+		return ret;
 
 	cmsg = CMSG_FIRSTHDR(&msg);
 
@@ -305,15 +286,13 @@ int lxc_abstract_unix_rcv_credential(int fd, void *data, size_t size)
 	    cmsg->cmsg_level == SOL_SOCKET &&
 	    cmsg->cmsg_type == SCM_CREDENTIALS) {
 		memcpy(&cred, CMSG_DATA(cmsg), sizeof(cred));
-		if (cred.uid &&
-		    (cred.uid != getuid() || cred.gid != getgid())) {
-			INFO("Message denied for '%d/%d'", cred.uid, cred.gid);
-			errno = EACCES;
-			return -1;
-		}
+
+		if (cred.uid && (cred.uid != getuid() || cred.gid != getgid()))
+			return log_error_errno(-1, EACCES,
+					       "Message denied for '%d/%d'",
+					       cred.uid, cred.gid);
 	}
 
-out:
 	return ret;
 }
 
@@ -323,14 +302,14 @@ int lxc_unix_sockaddr(struct sockaddr_un *ret, const char *path)
 
 	len = strlen(path);
 	if (len == 0)
-		return minus_one_set_errno(EINVAL);
+		return ret_set_errno(-1, EINVAL);
 	if (path[0] != '/' && path[0] != '@')
-		return minus_one_set_errno(EINVAL);
+		return ret_set_errno(-1, EINVAL);
 	if (path[1] == '\0')
-		return minus_one_set_errno(EINVAL);
+		return ret_set_errno(-1, EINVAL);
 
 	if (len + 1 > sizeof(ret->sun_path))
-		return minus_one_set_errno(EINVAL);
+		return ret_set_errno(-1, EINVAL);
 
 	*ret = (struct sockaddr_un){
 	    .sun_family = AF_UNIX,
@@ -345,17 +324,16 @@ int lxc_unix_sockaddr(struct sockaddr_un *ret, const char *path)
 	return (int)(offsetof(struct sockaddr_un, sun_path) + len + 1);
 }
 
-int lxc_unix_connect(struct sockaddr_un *addr)
+int lxc_unix_connect_type(struct sockaddr_un *addr, int type)
 {
-	__do_close_prot_errno int fd = -EBADF;
+	__do_close int fd = -EBADF;
 	int ret;
 	ssize_t len;
 
-	fd = socket(AF_UNIX, SOCK_STREAM | SOCK_CLOEXEC, 0);
-	if (fd < 0) {
-		SYSERROR("Failed to open new AF_UNIX socket");
-		return -1;
-	}
+	fd = socket(AF_UNIX, type | SOCK_CLOEXEC, 0);
+	if (fd < 0)
+		return log_error_errno(-1, errno,
+				       "Failed to open new AF_UNIX socket");
 
 	if (addr->sun_path[0] == '\0')
 		len = strlen(&addr->sun_path[1]);
@@ -364,14 +342,18 @@ int lxc_unix_connect(struct sockaddr_un *addr)
 
 	ret = connect(fd, (struct sockaddr *)addr,
 		      offsetof(struct sockaddr_un, sun_path) + len);
-	if (ret < 0) {
-		SYSERROR("Failed to bind new AF_UNIX socket");
-		return -1;
-	}
+	if (ret < 0)
+		return log_error_errno(-1, errno,
+				       "Failed to bind new AF_UNIX socket");
 
 	return move_fd(fd);
 }
 
+int lxc_unix_connect(struct sockaddr_un *addr, int type)
+{
+	return lxc_unix_connect_type(addr, SOCK_STREAM);
+}
+
 int lxc_socket_set_timeout(int fd, int rcv_timeout, int snd_timeout)
 {
 	struct timeval out = {0};
