@@ -1,26 +1,8 @@
-/*
- *
- * Copyright © 2013 Serge Hallyn <serge.hallyn@ubuntu.com>.
- * Copyright © 2013 Canonical Ltd.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+/* SPDX-License-Identifier: LGPL-2.1+ */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE 1
 #endif
-#include <alloca.h>
 #include <arpa/inet.h>
 #include <ctype.h>
 #include <errno.h>
@@ -49,6 +31,7 @@
 
 #include "compiler.h"
 #include "config.h"
+#include "file_utils.h"
 #include "log.h"
 #include "memory_utils.h"
 #include "network.h"
@@ -70,6 +53,14 @@
 
 #define usernic_error(format, ...) usernic_debug_stream(stderr, format, __VA_ARGS__)
 
+#define cmd_error_errno(__ret__, __errno__, format, ...)      \
+	({                                                    \
+		typeof(__ret__) __internal_ret__ = (__ret__); \
+		errno = (__errno__);                          \
+		CMD_SYSERROR(format, ##__VA_ARGS__);          \
+		__internal_ret__;                             \
+	})
+
 __noreturn static void usage(bool fail)
 {
 	fprintf(stderr, "Description:\n");
@@ -90,11 +81,11 @@ __noreturn static void usage(bool fail)
 
 static int open_and_lock(const char *path)
 {
-	__do_close_prot_errno int fd = -EBADF;
+	__do_close int fd = -EBADF;
 	int ret;
 	struct flock lk;
 
-	fd = open(path, O_RDWR | O_CREAT, S_IWUSR | S_IRUSR);
+	fd = open(path, O_RDWR | O_CREAT, S_IWUSR | S_IRUSR | O_CLOEXEC);
 	if (fd < 0) {
 		CMD_SYSERROR("Failed to open \"%s\"\n", path);
 		return -1;
@@ -340,7 +331,7 @@ static int get_alloted(char *me, char *intype, char *link,
 	int count = 0;
 	size_t len = 0;
 
-	fin = fopen(LXC_USERNIC_CONF, "r");
+	fin = fopen(LXC_USERNIC_CONF, "re");
 	if (!fin) {
 		CMD_SYSERROR("Failed to open \"%s\"\n", LXC_USERNIC_CONF);
 		return -1;
@@ -444,7 +435,8 @@ static char *find_line(char *buf_start, char *buf_end, char *name,
 		if (strncmp(buf_start, name, strlen(name)))
 			*found = false;
 		else
-			*owner = true;
+			if (strlen(name) == (size_t)(end_of_word - buf_start))
+				*owner = true;
 
 		buf_start = end_of_word + 1;
 		while ((buf_start < buf_end) && isblank(*buf_start))
@@ -503,25 +495,25 @@ static char *find_line(char *buf_start, char *buf_end, char *name,
 	return NULL;
 }
 
-static int instantiate_veth(char *veth1, char *veth2)
+static int instantiate_veth(char *veth1, char *veth2, pid_t pid, unsigned int mtu)
 {
 	int ret;
 
-	ret = lxc_veth_create(veth1, veth2);
+	ret = lxc_veth_create(veth1, veth2, pid, mtu);
 	if (ret < 0) {
-		errno = -ret;
 		CMD_SYSERROR("Failed to create %s-%s\n", veth1, veth2);
-		return -1;
+		return ret_errno(-ret);
 	}
 
-	/* Changing the high byte of the mac address to 0xfe, the bridge
+	/*
+	 * Changing the high byte of the mac address to 0xfe, the bridge
 	 * interface will always keep the host's mac address and not take the
 	 * mac address of a container.
 	 */
 	ret = setup_private_host_hw_addr(veth1);
 	if (ret < 0) {
-		errno = -ret;
 		CMD_SYSERROR("Failed to change mac address of host interface %s\n", veth1);
+		return ret_errno(-ret);
 	}
 
 	return netdev_set_flag(veth1, IFF_UP);
@@ -540,8 +532,9 @@ static int get_mtu(char *name)
 
 static int create_nic(char *nic, char *br, int pid, char **cnic)
 {
+	unsigned int mtu = 1500;
+	int ret;
 	char veth1buf[IFNAMSIZ], veth2buf[IFNAMSIZ];
-	int mtu, ret;
 
 	ret = snprintf(veth1buf, IFNAMSIZ, "%s", nic);
 	if (ret < 0 || ret >= IFNAMSIZ) {
@@ -555,16 +548,19 @@ static int create_nic(char *nic, char *br, int pid, char **cnic)
 		return -1;
 	}
 
+	if (strcmp(br, "none"))
+		mtu = get_mtu(br);
+	if (!mtu)
+		mtu = 1500;
+
 	/* create the nics */
-	ret = instantiate_veth(veth1buf, veth2buf);
+	ret = instantiate_veth(veth1buf, veth2buf, pid, mtu);
 	if (ret < 0) {
 		usernic_error("%s", "Error creating veth tunnel\n");
 		return -1;
 	}
 
 	if (strcmp(br, "none")) {
-		/* copy the bridge's mtu to both ends */
-		mtu = get_mtu(br);
 		if (mtu > 0) {
 			ret = lxc_netdev_set_mtu(veth1buf, mtu);
 			if (ret < 0) {
@@ -572,13 +568,6 @@ static int create_nic(char *nic, char *br, int pid, char **cnic)
 					      mtu, veth1buf);
 				goto out_del;
 			}
-
-			ret = lxc_netdev_set_mtu(veth2buf, mtu);
-			if (ret < 0) {
-				usernic_error("Failed to set mtu to %d on %s\n",
-					      mtu, veth2buf);
-				goto out_del;
-			}
 		}
 
 		/* attach veth1 to bridge */
@@ -589,14 +578,6 @@ static int create_nic(char *nic, char *br, int pid, char **cnic)
 		}
 	}
 
-	/* pass veth2 to target netns */
-	ret = lxc_netdev_move_by_name(veth2buf, pid, NULL);
-	if (ret < 0) {
-		usernic_error("Error moving %s to network namespace of %d\n",
-			      veth2buf, pid);
-		goto out_del;
-	}
-
 	*cnic = strdup(veth2buf);
 	if (!*cnic) {
 		usernic_error("Failed to copy string \"%s\"\n", veth2buf);
@@ -619,40 +600,35 @@ struct entry_line {
 static bool cull_entries(int fd, char *name, char *net_type, char *net_link,
 			 char *net_dev, bool *found_nicname)
 {
+	__do_free char *buf = NULL;
 	__do_free struct entry_line *entry_lines = NULL;
-	int i, ret;
-	char *buf, *buf_end, *buf_start;
-	struct stat sb;
 	int n = 0;
+	size_t length = 0;
+	int ret;
+	char *buf_end, *buf_start;
 	bool found, keep;
 
-	ret = fstat(fd, &sb);
+	ret = fd_to_buf(fd, &buf, &length);
 	if (ret < 0) {
-		CMD_SYSERROR("Failed to fstat\n");
+		CMD_SYSERROR("Failed to read database file\n");
 		return false;
 	}
-
-	if (!sb.st_size)
+	if (lseek(fd, 0, SEEK_SET) < 0)
 		return false;
 
-	buf = lxc_strmmap(NULL, sb.st_size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
-	if (buf == MAP_FAILED) {
-		CMD_SYSERROR("Failed to establish shared memory mapping\n");
+	if (length == 0)
 		return false;
-	}
 
 	buf_start = buf;
-	buf_end = buf + sb.st_size;
+	buf_end = buf + length;
 	while ((buf_start = find_line(buf_start, buf_end, name, net_type,
 				      net_link, net_dev, &(bool){true}, &found,
 				      &keep))) {
 		struct entry_line *newe;
 
 		newe = realloc(entry_lines, sizeof(*entry_lines) * (n + 1));
-		if (!newe) {
-			lxc_strmunmap(buf, sb.st_size);
+		if (!newe)
 			return false;
-		}
 
 		if (found)
 			*found_nicname = true;
@@ -670,7 +646,7 @@ static bool cull_entries(int fd, char *name, char *net_type, char *net_link,
 
 	buf_start = buf;
 
-	for (i = 0; i < n; i++) {
+	for (int i = 0; i < n; i++) {
 		if (!entry_lines[i].keep)
 			continue;
 
@@ -680,12 +656,7 @@ static bool cull_entries(int fd, char *name, char *net_type, char *net_link,
 		buf_start++;
 	}
 
-	ret = ftruncate(fd, buf_start - buf);
-	lxc_strmunmap(buf, sb.st_size);
-	if (ret < 0)
-		CMD_SYSERROR("Failed to set new file size\n");
-
-	return true;
+	return ftruncate(fd, buf_start - buf) == 0;
 }
 
 static int count_entries(char *buf, off_t len, char *name, char *net_type, char *net_link)
@@ -712,14 +683,13 @@ static int count_entries(char *buf, off_t len, char *name, char *net_type, char
 static char *get_nic_if_avail(int fd, struct alloted_s *names, int pid,
 			      char *intype, char *br, int allowed, char **cnic)
 {
-	__do_free char *newline = NULL;
+	__do_free char *buf = NULL, *newline = NULL;
+	size_t length = 0;
 	int ret;
 	size_t slen;
 	char *owner;
 	char nicname[IFNAMSIZ];
-	struct stat sb;
 	struct alloted_s *n;
-	char *buf = NULL;
 	uid_t uid;
 
 	for (n = names; n != NULL; n = n->next)
@@ -730,49 +700,48 @@ static char *get_nic_if_avail(int fd, struct alloted_s *names, int pid,
 
 	owner = names->name;
 
-	ret = fstat(fd, &sb);
+	ret = fd_to_buf(fd, &buf, &length);
 	if (ret < 0) {
-		CMD_SYSERROR("Failed to fstat\n");
-		return NULL;
+		CMD_SYSERROR("Failed to read database file\n");
+		return false;
 	}
+	if (lseek(fd, 0, SEEK_SET) < 0)
+		return false;
 
-	if (sb.st_size > 0) {
-		buf = lxc_strmmap(NULL, sb.st_size, PROT_READ | PROT_WRITE,
-				  MAP_SHARED, fd, 0);
-		if (buf == MAP_FAILED) {
-			CMD_SYSERROR("Failed to establish shared memory mapping\n");
-			return NULL;
-		}
-
+	if (length > 0) {
 		owner = NULL;
 
 		for (n = names; n != NULL; n = n->next) {
 			int count;
 
-			count = count_entries(buf, sb.st_size, n->name, intype, br);
+			count = count_entries(buf, length, n->name, intype, br);
 			if (count >= n->allowed)
 				continue;
 
 			owner = n->name;
 			break;
 		}
-
-		lxc_strmunmap(buf, sb.st_size);
 	}
 
 	if (!owner)
 		return NULL;
 
         uid = getuid();
-	/* for POSIX integer uids the network device name schema is vethUID_XXXXX */
+	/*
+	 * For POSIX integer uids the network device name schema is
+	 * vethUID_XXXX.
+	 * With four random characters passed to
+	 * lxc_ifname_alnum_case_sensitive() we get 62^4 = 14776336
+	 * combinations per uid. That's plenty of network devices for now.
+	 */
 	if (uid > 0 && uid <= 65536)
-		ret = snprintf(nicname, sizeof(nicname), "veth%d_XXXXX", uid);
+		ret = snprintf(nicname, sizeof(nicname), "veth%d_XXXX", uid);
 	else
 		ret = snprintf(nicname, sizeof(nicname), "vethXXXXXX");
 	if (ret < 0 || (size_t)ret >= sizeof(nicname))
 		return NULL;
 
-	if (!lxc_mkifname(nicname))
+	if (!lxc_ifname_alnum_case_sensitive(nicname))
 		return NULL;
 
 	ret = create_nic(nicname, br, pid, cnic);
@@ -814,17 +783,8 @@ static char *get_nic_if_avail(int fd, struct alloted_s *names, int pid,
 		return NULL;
 	}
 
-	/* Note that the file needs to be truncated to the size **without** the
-	 * \0 byte! Files are not \0-terminated!
-	 */
-	ret = ftruncate(fd, sb.st_size + slen);
-	if (ret < 0)
-		CMD_SYSERROR("Failed to truncate file\n");
-
-	buf = lxc_strmmap(NULL, sb.st_size + slen, PROT_READ | PROT_WRITE,
-			  MAP_SHARED, fd, 0);
-	if (buf == MAP_FAILED) {
-		CMD_SYSERROR("Failed to establish shared memory mapping\n");
+	if (lxc_pwrite_nointr(fd, newline, slen, length) != slen) {
+		CMD_SYSERROR("Failed to append new entry \"%s\" to database file", newline);
 
 		if (lxc_netdev_delete_by_name(nicname) != 0)
 			usernic_error("Error unlinking %s\n", nicname);
@@ -832,11 +792,15 @@ static char *get_nic_if_avail(int fd, struct alloted_s *names, int pid,
 		return NULL;
 	}
 
-	/* Note that the memory needs to be moved in the buffer **without** the
-	 * \0 byte! Files are not \0-terminated!
-	 */
-	memmove(buf + sb.st_size, newline, slen);
-	lxc_strmunmap(buf, sb.st_size + slen);
+	ret = ftruncate(fd, length + slen);
+	if (ret < 0) {
+		CMD_SYSERROR("Failed to truncate file\n");
+
+		if (lxc_netdev_delete_by_name(nicname) != 0)
+			usernic_error("Error unlinking %s\n", nicname);
+
+		return NULL;
+	}
 
 	return strdup(nicname);
 }
@@ -876,113 +840,84 @@ again:
 static char *lxc_secure_rename_in_ns(int pid, char *oldname, char *newname,
 				     int *container_veth_ifidx)
 {
-	int ofd, ret;
+	__do_close int fd = -EBADF, ofd = -EBADF;
+	int fret = -1;
+	int ifindex, ret;
 	pid_t pid_self;
 	uid_t ruid, suid, euid;
 	char ifname[IFNAMSIZ];
-	char *string_ret = NULL, *name = NULL;
-	int fd = -1, ifindex = -1;
 
 	pid_self = lxc_raw_getpid();
 
 	ofd = lxc_preserve_ns(pid_self, "net");
-	if (ofd < 0) {
-		usernic_error("Failed opening network namespace path for %d", pid_self);
-		return NULL;
-	}
+	if (ofd < 0)
+		return cmd_error_errno(NULL, errno, "Failed opening network namespace path for %d", pid_self);
 
 	fd = lxc_preserve_ns(pid, "net");
-	if (fd < 0) {
-		usernic_error("Failed opening network namespace path for %d", pid);
-		goto do_partial_cleanup;
-	}
+	if (fd < 0)
+		return cmd_error_errno(NULL, errno, "Failed opening network namespace path for %d", pid);
 
 	ret = getresuid(&ruid, &euid, &suid);
-	if (ret < 0) {
-		CMD_SYSERROR("Failed to retrieve real, effective, and saved user IDs\n");
-		goto do_partial_cleanup;
-	}
+	if (ret < 0)
+		return cmd_error_errno(NULL, errno, "Failed to retrieve real, effective, and saved user IDs\n");
 
 	ret = setns(fd, CLONE_NEWNET);
-	close(fd);
-	fd = -1;
-	if (ret < 0) {
-		CMD_SYSERROR("Failed to setns() to the network namespace of the container with PID %d\n",
-			     pid);
-		goto do_partial_cleanup;
-	}
+	if (ret < 0)
+		return cmd_error_errno(NULL, errno, "Failed to setns() to the network namespace of the container with PID %d\n", pid);
 
 	ret = setresuid(ruid, ruid, 0);
 	if (ret < 0) {
-		CMD_SYSERROR("Failed to drop privilege by setting effective user id and real user id to %d, and saved user ID to 0\n",
-			     ruid);
-		/* It's ok to jump to do_full_cleanup here since setresuid()
-		 * will succeed when trying to set real, effective, and saved to
-		 * values they currently have.
+		CMD_SYSERROR("Failed to drop privilege by setting effective user id and real user id to %d, and saved user ID to 0\n", ruid);
+		/*
+		 * It's ok to jump to do_full_cleanup here since setresuid()
+		 * will succeed when trying to set real, effective, and saved
+		 * to values they currently have.
 		 */
-		goto do_full_cleanup;
+		goto out_setns;
 	}
 
 	/* Check if old interface exists. */
 	ifindex = if_nametoindex(oldname);
 	if (!ifindex) {
 		CMD_SYSERROR("Failed to get netdev index\n");
-		goto do_full_cleanup;
+		goto out_setresuid;
 	}
 
-	/* When the IFLA_IFNAME attribute is passed something like "<prefix>%d"
+	/*
+	 * When the IFLA_IFNAME attribute is passed something like "<prefix>%d"
 	 * netlink will replace the format specifier with an appropriate index.
 	 * So we pass "eth%d".
 	 */
-	if (newname)
-		name = newname;
-	else
-		name = "eth%d";
-
-	ret = lxc_netdev_rename_by_name(oldname, name);
-	name = NULL;
+	ret = lxc_netdev_rename_by_name(oldname, newname ? newname : "eth%d");
 	if (ret < 0) {
-		usernic_error("Error %d renaming netdev %s to %s in container\n",
-		              ret, oldname, newname ? newname : "eth%d");
-		goto do_full_cleanup;
+		CMD_SYSERROR("Error %d renaming netdev %s to %s in container\n", ret, oldname, newname ? newname : "eth%d");
+		goto out_setresuid;
 	}
 
 	/* Retrieve new name for interface. */
 	if (!if_indextoname(ifindex, ifname)) {
 		CMD_SYSERROR("Failed to get new netdev name\n");
-		goto do_full_cleanup;
+		goto out_setresuid;
 	}
 
-	/* Allocation failure for strdup() is checked below. */
-	name = strdup(ifname);
-	string_ret = name;
-	*container_veth_ifidx = ifindex;
+	fret = 0;
 
-do_full_cleanup:
+out_setresuid:
 	ret = setresuid(ruid, euid, suid);
-	if (ret < 0) {
-		CMD_SYSERROR("Failed to restore privilege by setting effective user id to %d, real user id to %d, and saved user ID to %d\n",
-			     ruid, euid, suid);
-
-		string_ret = NULL;
-	}
+	if (ret < 0)
+		return cmd_error_errno(NULL, errno, "Failed to restore privilege by setting effective user id to %d, real user id to %d, and saved user ID to %d\n",
+				       ruid, euid, suid);
 
+out_setns:
 	ret = setns(ofd, CLONE_NEWNET);
-	if (ret < 0) {
-		CMD_SYSERROR("Failed to setns() to original network namespace of PID %d\n", ofd);
-		string_ret = NULL;
-	}
+	if (ret < 0)
+		return cmd_error_errno(NULL, errno, "Failed to setns() to original network namespace of PID %d\n", ofd);
 
-do_partial_cleanup:
-	if (fd >= 0)
-		close(fd);
+	if (fret < 0)
+		return NULL;
 
-	if (!string_ret && name)
-		free(name);
-
-	close(ofd);
-
-	return string_ret;
+	*container_veth_ifidx = ifindex;
+	return strdup(ifname);
 }
 
 /* If the caller (real uid, not effective uid) may read the /proc/[pid]/ns/net,
