@@ -1,22 +1,4 @@
-/* liblxcapi
- *
- * Copyright © 2019 Christian Brauner <christian.brauner@ubuntu.com>.
- * Copyright © 2019 Canonical Ltd.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
-
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
-
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- */
+/* SPDX-License-Identifier: LGPL-2.1+ */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE 1
@@ -57,33 +39,25 @@ int lxc_cmd_sock_rcv_state(int state_client_fd, int timeout)
 		out.tv_sec = timeout;
 		ret = setsockopt(state_client_fd, SOL_SOCKET, SO_RCVTIMEO,
 				(const void *)&out, sizeof(out));
-		if (ret < 0) {
-			SYSERROR("Failed to set %ds timeout on container "
-				 "state socket",
-				 timeout);
-			return -1;
-		}
+		if (ret < 0)
+			return log_error_errno(-1, errno, "Failed to set %ds timeout on container state socket", timeout);
 	}
 
 	memset(&msg, 0, sizeof(msg));
 
 	ret = lxc_recv_nointr(state_client_fd, &msg, sizeof(msg), 0);
-	if (ret < 0) {
-		SYSERROR("Failed to receive message");
-		return -1;
-	}
+	if (ret < 0)
+		return log_error_errno(-1, errno, "Failed to receive message");
 
-	TRACE("Received state %s from state client %d",
-	      lxc_state2str(msg.value), state_client_fd);
-
-	return msg.value;
+	return log_trace(msg.value, "Received state %s from state client %d",
+			 lxc_state2str(msg.value), state_client_fd);
 }
 
 /* Register a new state client and retrieve state from command socket. */
 int lxc_cmd_sock_get_state(const char *name, const char *lxcpath,
 			   lxc_state_t states[MAX_STATE], int timeout)
 {
-	__do_close_prot_errno int state_client_fd = -EBADF;
+	__do_close int state_client_fd = -EBADF;
 	int ret;
 
 	ret = lxc_cmd_add_state_client(name, lxcpath, states, &state_client_fd);
@@ -128,43 +102,36 @@ int lxc_make_abstract_socket_name(char *path, size_t pathlen,
 
 	if (hashed_sock_name != NULL) {
 		ret = snprintf(offset, len, "lxc/%s/%s", hashed_sock_name, suffix);
-		if (ret < 0 || ret >= len) {
-			ERROR("Failed to create abstract socket name");
-			return -1;
-		}
+		if (ret < 0 || (size_t)ret >= len)
+			return log_error_errno(-1, errno, "Failed to create abstract socket name");
 		return 0;
 	}
 
 	if (!lxcpath) {
 		lxcpath = lxc_global_config_value("lxc.lxcpath");
-		if (!lxcpath) {
-			ERROR("Failed to allocate memory");
-			return -1;
-		}
+		if (!lxcpath)
+			return log_error(-1, "Failed to allocate memory");
 	}
 
 	ret = snprintf(offset, len, "%s/%s/%s", lxcpath, name, suffix);
-	if (ret < 0) {
-		ERROR("Failed to create abstract socket name");
-		return -1;
-	}
-	if (ret < len)
-		return 0;
+	if (ret < 0)
+		return log_error_errno(-1, errno, "Failed to create abstract socket name");
 
-	/* ret >= len; lxcpath or name is too long.  hash both */
-	tmplen = strlen(name) + strlen(lxcpath) + 2;
-	tmppath = must_realloc(NULL, tmplen);
-	ret = snprintf(tmppath, tmplen, "%s/%s", lxcpath, name);
-	if (ret < 0 || (size_t)ret >= tmplen) {
-		ERROR("Failed to create abstract socket name");
-		return -1;
-	}
+	/*
+	 * ret >= len. This means lxcpath and name are too long. We need to
+	 * hash both.
+	 */
+	if (ret >= len) {
+		tmplen = strlen(name) + strlen(lxcpath) + 2;
+		tmppath = must_realloc(NULL, tmplen);
+		ret = snprintf(tmppath, tmplen, "%s/%s", lxcpath, name);
+		if (ret < 0 || (size_t)ret >= tmplen)
+			return log_error_errno(-1, errno, "Failed to create abstract socket name");
 
-	hash = fnv_64a_buf(tmppath, ret, FNV1A_64_INIT);
-	ret = snprintf(offset, len, "lxc/%016" PRIx64 "/%s", hash, suffix);
-	if (ret < 0 || ret >= len) {
-		ERROR("Failed to create abstract socket name");
-		return -1;
+		hash = fnv_64a_buf(tmppath, ret, FNV1A_64_INIT);
+		ret = snprintf(offset, len, "lxc/%016" PRIx64 "/%s", hash, suffix);
+		if (ret < 0 || (size_t)ret >= len)
+			return log_error_errno(-1, errno, "Failed to create abstract socket name");
 	}
 
 	return 0;
@@ -216,8 +183,8 @@ int lxc_add_state_client(int state_client_fd, struct lxc_handler *handler,
 		return state;
 	}
 
-	TRACE("Added state client %d to state client list", state_client_fd);
 	move_ptr(newclient);
 	move_ptr(tmplist);
+	TRACE("Added state client fd %d to state client list", state_client_fd);
 	return MAX_STATE;
 }
